@@ -13,6 +13,7 @@
 # Copyright (C) 2015, 2018-2020 Philipp Wolfer
 # Copyright (C) 2016-2018 Sambhav Kothari
 # Copyright (C) 2018 Vishal Choudhary
+# Copyright (C) 2020 Gabriel Ferreira
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -232,6 +233,8 @@ class MetadataBox(QtWidgets.QTableWidget):
         # TR: Keyboard shortcut for "Remove" (tag)
         self.remove_tag_shortcut = QtWidgets.QShortcut(QtGui.QKeySequence(_("Alt+Shift+R")), self, self.remove_selected_tags)
         self.preserved_tags = PreservedTags()
+        self._single_file_album = False
+        self._single_track_album = False
 
     def get_file_lookup(self):
         """Return a FileLookup object."""
@@ -249,6 +252,7 @@ class MetadataBox(QtWidgets.QTableWidget):
             "musicbrainz_artistid": lookup.artist_lookup,
             "musicbrainz_albumartistid": lookup.artist_lookup,
             "musicbrainz_releasegroupid": lookup.release_group_lookup,
+            "musicbrainz_discid": lookup.discid_lookup,
             "acoustid_id": lookup.acoust_lookup
         }
         return LOOKUP_TAGS
@@ -290,6 +294,7 @@ class MetadataBox(QtWidgets.QTableWidget):
                     self.clipboard = list(self.tag_diff.new[tag])
             elif e.key() == QtCore.Qt.Key_V and column == 2 and tag != "~length":
                 self.set_tag_values(tag, list(self.clipboard))
+                self.update()
         return super().event(e)
 
     def closeEditor(self, editor, hint):
@@ -302,7 +307,7 @@ class MetadataBox(QtWidgets.QTableWidget):
         else:
             self.set_tag_values(tag, new)
         self.editing = None
-        self.update()
+        self.update(drop_album_caches=tag == 'album')
 
     @staticmethod
     def _get_editor_value(editor):
@@ -349,21 +354,34 @@ class MetadataBox(QtWidgets.QTableWidget):
                         removals.append(partial(self.remove_tag, tag))
                     status = self.tag_diff.status[tag] & TagStatus.CHANGED
                     if status == TagStatus.CHANGED or status == TagStatus.REMOVED:
+                        file_tracks = []
+                        track_albums = set()
                         for file in self.files:
                             objects = [file]
                             if file.parent in self.tracks and len(self.files & set(file.parent.linked_files)) == 1:
                                 objects.append(file.parent)
+                                file_tracks.append(file.parent)
+                                track_albums.add(file.parent.album)
                             orig_values = list(file.orig_metadata.getall(tag)) or [""]
                             useorigs.append(partial(self.set_tag_values, tag, orig_values, objects))
+                        for track in set(self.tracks)-set(file_tracks):
+                            objects = [track]
+                            orig_values = list(track.orig_metadata.getall(tag)) or [""]
+                            useorigs.append(partial(self.set_tag_values, tag, orig_values, objects))
+                            track_albums.add(track.album)
+                        for album in track_albums:
+                            objects = [album]
+                            orig_values = list(album.orig_metadata.getall(tag)) or [""]
+                            useorigs.append(partial(self.set_tag_values, tag, orig_values, objects))
                 if removals:
                     remove_tag_action = QtWidgets.QAction(_("Remove"), self.parent)
-                    remove_tag_action.triggered.connect(lambda: [f() for f in removals])
+                    remove_tag_action.triggered.connect(partial(self._apply_update_funcs, removals))
                     remove_tag_action.setShortcut(self.remove_tag_shortcut.key())
                     menu.addAction(remove_tag_action)
                 if useorigs:
                     name = ngettext("Use Original Value", "Use Original Values", len(useorigs))
                     use_orig_value_action = QtWidgets.QAction(name, self.parent)
-                    use_orig_value_action.triggered.connect(lambda: [f() for f in useorigs])
+                    use_orig_value_action.triggered.connect(partial(self._apply_update_funcs, useorigs))
                     menu.addAction(use_orig_value_action)
                     menu.addSeparator()
             if len(tags) == 1 or removals or useorigs:
@@ -374,6 +392,11 @@ class MetadataBox(QtWidgets.QTableWidget):
         menu.exec_(event.globalPos())
         event.accept()
 
+    def _apply_update_funcs(self, funcs):
+        for f in funcs:
+            f()
+        self.parent.update_selection(new_selection=False, drop_album_caches=True)
+
     def edit_tag(self, tag):
         EditTagDialog(self.parent, tag).exec_()
 
@@ -395,11 +418,11 @@ class MetadataBox(QtWidgets.QTableWidget):
         if not values and self.tag_is_removable(tag):
             for obj in objects:
                 del obj.metadata[tag]
+                obj.update()
         elif values:
             for obj in objects:
                 obj.metadata[tag] = values
                 obj.update()
-        self.update()
         self.parent.ignore_selection_changes = False
 
     def remove_tag(self, tag):
@@ -409,6 +432,7 @@ class MetadataBox(QtWidgets.QTableWidget):
         for tag in self.selected_tags(discard=('~length',)):
             if self.tag_is_removable(tag):
                 self.remove_tag(tag)
+        self.parent.update_selection(new_selection=False, drop_album_caches=True)
 
     def tag_is_removable(self, tag):
         return self.tag_diff.status[tag] & TagStatus.NOTREMOVABLE == 0
@@ -449,15 +473,16 @@ class MetadataBox(QtWidgets.QTableWidget):
         self.selection_mutex.unlock()
 
     @throttle(100)
-    def update(self):
+    def update(self, drop_album_caches=False):
         if self.editing:
             return
+        new_selection = self.selection_dirty
         if self.selection_dirty:
             self._update_selection()
-        thread.run_task(self._update_tags, self._update_items,
+        thread.run_task(partial(self._update_tags, new_selection, drop_album_caches), self._update_items,
             thread_pool=self.tagger.priority_thread_pool)
 
-    def _update_tags(self):
+    def _update_tags(self, new_selection=True, drop_album_caches=False):
         self.selection_mutex.lock()
         files = self.files
         tracks = self.tracks
@@ -466,6 +491,24 @@ class MetadataBox(QtWidgets.QTableWidget):
         if not (files or tracks):
             return None
 
+        if new_selection or drop_album_caches:
+            self._single_file_album = len(set([file.metadata["album"] for file in files])) == 1
+            self._single_track_album = len(set([track.metadata["album"] for track in tracks])) == 1
+
+        while not new_selection:  # Just an if with multiple exit points
+            # If we are dealing with the same selection
+            # skip updates unless it we are dealing with a single file/track
+            if len(files) == 1:
+                break
+            if len(tracks) == 1:
+                break
+            # Or if we are dealing with a single cluster/album
+            if self._single_file_album:
+                break
+            if self._single_track_album:
+                break
+            return self.tag_diff
+
         self.colors = {
             TagStatus.NOCHANGE: self.palette().color(QtGui.QPalette.Text),
             TagStatus.REMOVED: QtGui.QBrush(interface_colors.get_qcolor('tagstatus_removed')),
@@ -506,7 +549,10 @@ class MetadataBox(QtWidgets.QTableWidget):
             if track.num_linked_files == 0:
                 for name, values in track.metadata.rawitems():
                     if not name.startswith("~"):
-                        tag_diff.add(name, values, values, True)
+                        if name in track.orig_metadata:
+                            tag_diff.add(name, [track.orig_metadata[name]], values, True)
+                        else:
+                            tag_diff.add(name, values, values, True)
 
                 length = str(track.metadata.length)
                 tag_diff.add("~length", length, length, False)
