@@ -1,7 +1,14 @@
 # -*- coding: utf-8 -*-
 #
 # Picard, the next-generation MusicBrainz tagger
-# Copyright (C) 2006 Lukáš Lalinský
+#
+# Copyright (C) 2006-2007 Lukáš Lalinský
+# Copyright (C) 2010, 2018-2019 Philipp Wolfer
+# Copyright (C) 2012, 2014 Wieland Hoffmann
+# Copyright (C) 2012-2014 Michael Wiencek
+# Copyright (C) 2013-2015, 2018-2019 Laurent Monin
+# Copyright (C) 2016-2018 Sambhav Kothari
+# Copyright (C) 2017 Suhas
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -17,11 +24,9 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
+
 from picard import config
-from picard.coverart.providers import (
-    cover_art_providers,
-    is_provider_enabled,
-)
+from picard.coverart.providers import cover_art_providers
 
 from picard.ui.checkbox_list_item import CheckboxListItem
 from picard.ui.moveable_list_view import MoveableListView
@@ -39,6 +44,7 @@ class CoverOptionsPage(OptionsPage):
     PARENT = None
     SORT_ORDER = 35
     ACTIVE = True
+    HELP_URL = '/config/options_cover.html'
 
     options = [
         config.BoolOption("setting", "save_images_to_tags", True),
@@ -67,14 +73,8 @@ class CoverOptionsPage(OptionsPage):
     def load_cover_art_providers(self):
         """Load available providers, initialize provider-specific options, restore state of each
         """
-        providers = cover_art_providers()
-        for provider in providers:
-            try:
-                title = _(provider.TITLE)
-            except AttributeError:
-                title = provider.NAME
-            checked = is_provider_enabled(provider.NAME)
-            self.ui.ca_providers_list.addItem(CheckboxListItem(title, checked=checked, data=provider.NAME))
+        for p in cover_art_providers():
+            self.ui.ca_providers_list.addItem(CheckboxListItem(_(p.title), checked=p.enabled, data=p.name))
 
     def restore_defaults(self):
         # Remove previous entries
@@ -110,14 +110,21 @@ class CoverOptionsPage(OptionsPage):
         self.update_filename()
         self.update_save_images_to_tags()
 
+    def update_ca_providers_groupbox_state(self):
+        files_enabled = self.ui.save_images_to_files.isChecked()
+        tags_enabled = self.ui.save_images_to_tags.isChecked()
+        self.ui.ca_providers_groupbox.setEnabled(files_enabled or tags_enabled)
+
     def update_filename(self):
         enabled = self.ui.save_images_to_files.isChecked()
         self.ui.cover_image_filename.setEnabled(enabled)
         self.ui.save_images_overwrite.setEnabled(enabled)
+        self.update_ca_providers_groupbox_state()
 
     def update_save_images_to_tags(self):
         enabled = self.ui.save_images_to_tags.isChecked()
         self.ui.cb_embed_front_only.setEnabled(enabled)
+        self.update_ca_providers_groupbox_state()
 
 
 register_options_page(CoverOptionsPage)
