@@ -20,6 +20,7 @@
 # Copyright (C) 2017 Frederik “Freso” S. Olesen
 # Copyright (C) 2018 Bob Swift
 # Copyright (C) 2018 Vishal Choudhary
+# Copyright (C) 2020 Ray Bouchard
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -38,6 +39,7 @@
 
 import builtins
 from collections import namedtuple
+from collections.abc import Mapping
 import html
 import json
 import ntpath
@@ -48,9 +50,11 @@ import sys
 from time import time
 import unicodedata
 
+from dateutil.parser import parse
+
 from PyQt5 import QtCore
 
-# Required for compatibility with lastfmplus which imports this from here rather than loading it direct.
+from picard import log
 from picard.const import MUSICBRAINZ_SERVERS
 from picard.const.sys import (
     FROZEN_TEMP_PATH,
@@ -88,12 +92,11 @@ _io_encoding = sys.getfilesystemencoding()
 
 
 # The following was adapted from k3b's source code:
-#// On a glibc system the system locale defaults to ANSI_X3.4-1968
-#// It is very unlikely that one would set the locale to ANSI_X3.4-1968
-#// intentionally
+# On a glibc system the system locale defaults to ANSI_X3.4-1968
+# It is very unlikely that one would set the locale to ANSI_X3.4-1968
+# intentionally
 def check_io_encoding():
     if _io_encoding == "ANSI_X3.4-1968":
-        from picard import log
         log.warning("""
 System locale charset is ANSI_X3.4-1968
 Your system's locale charset (i.e. the charset used to encode filenames)
@@ -319,16 +322,16 @@ def uniqify(seq):
 _tracknum_regexps = (
     # search for explicit track number (prefix "track")
     r"track[\s_-]*(?:no|nr)?[\s_-]*(\d+)",
-    # search for 2-digit number at start of string
-    r"^(\d{2})\D?",
-    # search for 2-digit number at end of string
-    r"\D?(\d{2})$",
+    # search for 2-digit number at start of string
+    r"^(\d{2})\D",
+    # search for 2-digit number at end of string
+    r"\D(\d{2})$",
 )
 
 
 def tracknum_from_filename(base_filename):
     """Guess and extract track number from filename
-    Returns -1 if none found, the number as integer else
+    Returns `None` if none found, the number as integer else
     """
     filename, _ = os.path.splitext(base_filename)
     for r in _tracknum_regexps:
@@ -344,7 +347,7 @@ def tracknum_from_filename(base_filename):
                       0 < int(n) <= 99])
     if numbers:
         return numbers[0]
-    return -1
+    return None
 
 
 # Provide os.path.samefile equivalent which is missing in Python under Windows
@@ -501,7 +504,6 @@ def __convert_to_string(obj):
 
 
 def convert_to_string(obj):
-    from picard import log
     log.warning("string_() and convert_to_string() are deprecated, do not use")
     return __convert_to_string(obj)
 
@@ -645,3 +647,15 @@ def limited_join(a_list, limit, join_string='+', middle_string='…'):
     start = a_list[:half]
     end = a_list[-half:]
     return join_string.join(start + [middle_string] + end)
+
+
+def extract_year_from_date(dt):
+    """ Extracts year from  passed in date either dict or string """
+
+    try:
+        if isinstance(dt, Mapping):
+            return int(dt.get('year'))
+        else:
+            return parse(dt).year
+    except (TypeError, ValueError):
+        return None
